#! /usr/bin/env python
# coding=utf-8
"""tool - A simple pyisbn interface for the command line"""
# Copyright © 2007-2013  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# You won't find this pretty… however it was quick to write and it's compatible
# with 2.4-3.3

import argparse
import sys

from pyisbn import (Isbn, URL_MAP, _version)


def isbn_typecheck(string):
    try:
        isbn = Isbn(string)
        if not isbn.validate():
            raise ValueError('Invalid checksum')
    except ValueError:
        ex = sys.exc_info()[1]
        raise argparse.ArgumentTypeError('%s %r' % (ex.args[0], string))
    return isbn


def partial_arg(f):
    def wrapper(*args, **kwargs):
        # Can't mix kwargs and supplied args in call for Python 2.5 compat
        kwargs.update({'dest': 'command', 'action': 'store_const'})
        if not 'const' in kwargs:
            kwargs['const'] = args[1][2:].replace('-', '_')
        return f(*args, **kwargs)
    return wrapper


def main():
    parser = argparse.ArgumentParser(
        description=__doc__.splitlines()[0].split('-', 1)[1],
        epilog='Please report bugs to jnrowe@gmail.com'
    )
    parser.add_argument('--version', action='version',
                        version='pyisbn %s' % _version.dotted)
    commands = parser.add_mutually_exclusive_group()
    parg = partial_arg(commands.add_argument)
    parg('-c', '--checksum', const='calculate_checksum',
         help='generate checksum')
    parg('-x', '--convert', help='convert between 10- and 13-digit types')
    commands.add_argument('-u', '--to-url',
                          choices=sorted(URL_MAP.keys()),
                          help='generate URL')
    parg('-n', '--to-urn', help='generate RFC 3187 URN')
    parser.add_argument('isbn', type=isbn_typecheck, nargs='+',
                        help='ISBNs to operate on')

    args = parser.parse_args()

    for isbn in args.isbn:
        if args.command:
            res = getattr(isbn, args.command)()
        elif args.to_url:
            res = isbn.to_url(args.to_url)
        print(res)


if __name__ == '__main__':
    main()



#
# coding=utf-8
"""test_isbn - Test Isbn class"""
# Copyright © 2007-2013  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from unittest import (TestCase, skipIf)
from sys import version_info

from expecter import expect
from nose2.tools import params

from pyisbn import (CountryError, Isbn, SiteError)


class TestIsbn(TestCase):
    @params(
        '9780521871723',
        '3540009787',
    )
    def test___repr__(self, isbn):
        expect(repr(Isbn(isbn))) == 'Isbn(%r)' % isbn

    @params(
        '9780521871723',
        '978-052-187-1723',
        '3540009787',
    )
    def test___str__(self, isbn):
        expect(str(Isbn(isbn))) == 'ISBN %s' % isbn

    @skipIf(version_info < (2, 6),
            "format() not supported with this Python version")
    @params(
        ('9780521871723', '', 'ISBN 9780521871723'),
        ('978-052-187-1723', 'urn', 'URN:ISBN:978-052-187-1723'),
        ('3540009787', 'url',
         'http://www.amazon.com/s?search-alias=stripbooks&field-isbn='
         '3540009787'),
        ('3540009787', 'url:amazon:uk',
         'http://www.amazon.co.uk/s?search-alias=stripbooks&field-isbn='
         '3540009787'),
    )
    def test___format__(self, isbn, format_spec, result):
        expect(format(Isbn(isbn), format_spec)) == result

    @params(
        ('978-052-187-1723', '3'),
        ('3540009787', '7'),
        ('354000978', '7'),
    )
    def test_calculate_checksum(self, isbn, result):
        expect(Isbn(isbn).calculate_checksum()) == result

    @params(
        ('0071148167', '9780071148160'),
        ('9780071148160', '0071148167'),
    )
    def test_convert(self, isbn, result):
        expect(Isbn(isbn).convert()) == result

    @params(
        ('978-052-187-1723', True),
        ('978-052-187-1720', False),
        ('3540009787', True),
        ('354000978x', False),
    )
    def test_validate(self, isbn, result):
        expect(Isbn(isbn).validate()) == result

    @params(
        ('us', 'http://www.amazon.com/s?search-alias=stripbooks&field-isbn=0071148167'),
        ('uk', 'http://www.amazon.co.uk/s?search-alias=stripbooks&field-isbn=0071148167'),
        ('de', 'http://www.amazon.de/s?search-alias=stripbooks&field-isbn=0071148167'),
    )
    def test_to_url(self, country, result):
        expect(Isbn('0071148167').to_url(country=country)) == result

    def test_to_url_invalid_country(self):
        with expect.raises(CountryError, "zh"):
            Isbn('0071148167').to_url(country='zh')

    @params(
        ('copac', 'http://copac.ac.uk/search?isn=0071148167'),
        ('google', 'http://books.google.com/books?vid=isbn:0071148167'),
        ('isbndb', 'http://isbndb.com/search/all?query=0071148167'),
        ('worldcat', 'http://worldcat.org/isbn/0071148167'),
        ('waterstones',
         'http://www.waterstones.com/waterstonesweb/advancedSearch.do?buttonClicked=2&isbn=0071148167'),
        ('whsmith',
         'http://www.whsmith.co.uk/CatalogAndSearch/SearchWithinCategory.aspx?as_ISBN=0071148167'),
    )
    def test_to_url_site(self, site, result):
        expect(Isbn('0071148167').to_url(site=site)) == result

    def test_to_url_invalid_site(self):
        with expect.raises(SiteError, "nosite"):
            Isbn('0071148167').to_url(site='nosite')

    def test_to_urn(self):
        expect(Isbn('0071148167').to_urn()) == 'URN:ISBN:0071148167'



#
# coding=utf-8
"""conf - Sphinx configuration information"""
# Copyright © 2007-2013  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys

from subprocess import (CalledProcessError, check_output)

root_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.insert(0, root_dir)

import pyisbn

extensions = \
    ['sphinx.ext.%s' % ext for ext in ['autodoc', 'coverage', 'doctest',
                                       'ifconfig', 'intersphinx', 'todo',
                                       'viewcode']] \
    + ["sphinxcontrib.%s" % ext for ext in ['cheeseshop']]

# Only activate spelling, if it is installed.  It is not required in the
# general case and we don't have the granularity to describe this in a clean
# way
try:
    from sphinxcontrib import spelling  # NOQA
except ImportError:
    pass
else:
    extensions.append('sphinxcontrib.spelling')

master_doc = 'index'
source_suffix = '.rst'

project = 'pyisbn'
try:
    unicode
    copyright = pyisbn.__copyright__.decode('utf-8')
except NameError:
    copyright = pyisbn.__copyright__

version = ".".join(map(str, pyisbn._version.tuple[:2]))
release = pyisbn._version.dotted

pygments_style = 'sphinx'
html_theme_options = {
    "externalrefs": True,
}
try:
    html_last_updated_fmt = check_output(['git', 'log',
                                          "--pretty=format:'%ad [%h]'",
                                          '--date=short', '-n1'])
except CalledProcessError:
    pass

# Autodoc extension settings
autoclass_content = "init"
autodoc_default_flags = ['members', 'inherited-members']

intersphinx_mapping = {
    'python': ('http://docs.python.org/', os.getenv('SPHINX_PYTHON_OBJECTS')),
}

spelling_lang = 'en_GB'
spelling_word_list_filename = 'wordlist.txt'
